@@ -3,7 +3,7 @@
  *
  * This file is part of msmtp, an SMTP client.
  *
- * Copyright (C) 2000, 2003, 2004, 2005
+ * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007
  * Martin Lambers <marlam@marlam.de>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -17,15 +17,12 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- *   msmtp is released under the GPL with the additional exemption that
- *   compiling, linking, and/or using OpenSSL is allowed.
+ *   along with this program; if not, write to the Free Software Foundation,
+ *   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+# include "config.h"
 #endif
 
 #include <stdlib.h>
@@ -34,13 +31,14 @@
 #include <string.h>
 #include <ctype.h>
 #include <errno.h>
-extern int errno;
 
+#include "gettext.h"
 #include "xalloc.h"
+#include "xvasprintf.h"
 
-#include "smtp.h"
-#include "paths.h"
 #include "list.h"
+#include "smtp.h"
+#include "tools.h"
 #include "conf.h"
 
 /* buffer size for configuration file lines */
@@ -62,9 +60,10 @@ account_t *account_new(const char *conffile, const char *id)
     a->mask = 0;
     a->host = NULL;
     a->port = 0;		/* this must be set later */
-    a->connect_timeout = 0;
+    a->timeout = 0;
     a->protocol = SMTP_PROTO_SMTP;
     a->domain = xstrdup("localhost");
+    a->auto_from = 0;
     a->from = NULL;
     a->maildomain = NULL;
     a->dsn_return = NULL;
@@ -75,11 +74,12 @@ account_t *account_new(const char *conffile, const char *id)
     a->password = NULL;
     a->ntlmdomain = NULL;
     a->tls = 0;
+    a->tls_nostarttls = 0;
     a->tls_key_file = NULL;
     a->tls_cert_file = NULL;
     a->tls_trust_file = NULL;
-    a->tls_nostarttls = 0;
     a->tls_nocertcheck = 0;
+    a->tls_force_sslv3 = 0;
     a->logfile = NULL;
     a->syslog = NULL;
     return a;
@@ -104,9 +104,10 @@ account_t *account_copy(account_t *acc)
 	a->mask = acc->mask;
 	a->host = acc->host ? xstrdup(acc->host) : NULL;
 	a->port = acc->port;
-	a->connect_timeout = acc->connect_timeout;
+	a->timeout = acc->timeout;
 	a->protocol = acc->protocol;
 	a->domain = acc->domain ? xstrdup(acc->domain) : NULL;
+	a->auto_from = acc->auto_from;
 	a->from = acc->from ? xstrdup(acc->from) : NULL;
 	a->maildomain = acc->maildomain ? xstrdup(acc->maildomain) : NULL;
 	a->dsn_return = acc->dsn_return ? xstrdup(acc->dsn_return) : NULL;
@@ -117,13 +118,14 @@ account_t *account_copy(account_t *acc)
 	a->password = acc->password ? xstrdup(acc->password) : NULL;
 	a->ntlmdomain = acc->ntlmdomain ? xstrdup(acc->ntlmdomain) : NULL;
 	a->tls = acc->tls;
+	a->tls_nostarttls = acc->tls_nostarttls;
 	a->tls_key_file = acc->tls_key_file ? xstrdup(acc->tls_key_file) : NULL;
 	a->tls_cert_file = 
 	    acc->tls_cert_file ? xstrdup(acc->tls_cert_file) : NULL;
 	a->tls_trust_file = 
 	    acc->tls_trust_file ? xstrdup(acc->tls_trust_file) : NULL;
-	a->tls_nostarttls = acc->tls_nostarttls;
 	a->tls_nocertcheck = acc->tls_nocertcheck;
+	a->tls_force_sslv3 = acc->tls_force_sslv3;
 	a->logfile = acc->logfile ? xstrdup(acc->logfile) : NULL;
 	a->syslog = acc->syslog ? xstrdup(acc->syslog) : NULL;
     }
@@ -244,8 +246,9 @@ int get_pos_int(const char *s)
     long x;
     char *p;
 
+    errno = 0;
     x = strtol(s, &p, 0);
-    if (p == s || x <= 0 || x == LONG_MAX || x > INT_MAX)
+    if (p == s || x <= 0 || (x == LONG_MAX && errno == ERANGE) || x > INT_MAX)
     {
    	x = -1;
     }
@@ -409,9 +412,9 @@ void override_account(account_t *acc1, account_t *acc2)
     {
 	acc1->port = acc2->port;
     }
-    if (acc2->mask & ACC_CONNECT_TIMEOUT)
+    if (acc2->mask & ACC_TIMEOUT)
     {
-	acc1->connect_timeout = acc2->connect_timeout;
+	acc1->timeout = acc2->timeout;
     }
     if (acc2->mask & ACC_PROTOCOL)
     {
@@ -422,6 +425,10 @@ void override_account(account_t *acc1, account_t *acc2)
 	free(acc1->domain);
 	acc1->domain = acc2->domain ? xstrdup(acc2->domain) : NULL;
     }
+    if (acc2->mask & ACC_AUTO_FROM)
+    {
+	acc1->auto_from = acc2->auto_from;
+    }
     if (acc2->mask & ACC_FROM)
     {
 	free(acc1->from);
@@ -456,6 +463,10 @@ void override_account(account_t *acc1, account_t *acc2)
     {
 	acc1->tls = acc2->tls;
     }
+    if (acc2->mask & ACC_TLS_NOSTARTTLS)
+    {
+	acc1->tls_nostarttls = acc2->tls_nostarttls;
+    }
     if (acc2->mask & ACC_TLS_KEY_FILE)
     {
 	free(acc1->tls_key_file);
@@ -474,9 +485,9 @@ void override_account(account_t *acc1, account_t *acc2)
 	acc1->tls_trust_file = 
 	    acc2->tls_trust_file ? xstrdup(acc2->tls_trust_file) : NULL;
     }
-    if (acc2->mask & ACC_TLS_NOSTARTTLS)
+    if (acc2->mask & ACC_TLS_FORCE_SSLV3)
     {
-	acc1->tls_nostarttls = acc2->tls_nostarttls;
+	acc1->tls_force_sslv3 = acc2->tls_force_sslv3;
     }
     if (acc2->mask & ACC_TLS_NOCERTCHECK)
     {
@@ -516,29 +527,37 @@ void override_account(account_t *acc1, account_t *acc2)
  * see conf.h
  */
 
-int check_account(account_t *acc, char *errstr)
+int check_account(account_t *acc, int sendmail_mode, char **errstr)
 {
     if (!acc->host)
     {
-	snprintf(errstr, errstr_bufsize, "no host information");
+	*errstr = xasprintf(_("host not set"));
+	return CONF_ESYNTAX;
+    }
+    if (acc->port == 0)
+    {
+	*errstr = xasprintf(_("port not set"));
+	return CONF_ESYNTAX;
+    }
+    if (sendmail_mode && !acc->from)
+    {
+	*errstr = xasprintf(_("envelope-from address is missing"));
 	return CONF_ESYNTAX;
     }
     if (acc->tls_key_file && !acc->tls_cert_file) 
     {
-	snprintf(errstr, errstr_bufsize, 
-		"TLS key file requires TLS cert file");
+	*errstr = xasprintf(_("tls_key_file requires tls_cert_file"));
 	return CONF_ESYNTAX;
     }
     if (!acc->tls_key_file && acc->tls_cert_file)
     {
-	snprintf(errstr, errstr_bufsize, 
-		"TLS cert file requires TLS key file");
+	*errstr = xasprintf(_("tls_cert_file requires tls_key_file"));
 	return CONF_ESYNTAX;
     }
     if (acc->tls_nocertcheck && acc->tls_trust_file)
     {
-	snprintf(errstr, errstr_bufsize, 
-		"cannot use TLS trust file with TLS cert check turned off");
+	*errstr = xasprintf(
+		_("cannot use tls_trust_file with tls_certcheck turned off"));
 	return CONF_ESYNTAX;
     }
 
@@ -550,9 +569,14 @@ int check_account(account_t *acc, char *errstr)
  * some small helper functions
  */
 
+int is_blank(int c)
+{
+    return (c == ' ' || c == '\t');
+}
+
 int skip_blanks(const char *s, int i)
 {
-    while (s[i] == ' ' || s[i] == '\t')
+    while (is_blank(s[i]))
     {
 	i++;
     }
@@ -563,7 +587,7 @@ int get_cmd_length(const char *s)
 {
     int i = 0;
     
-    while (s[i] != '\0' && s[i] != ' ' && s[i] != '\t')
+    while (s[i] != '\0' && !is_blank(s[i]))
     {
 	i++;
     }
@@ -576,7 +600,7 @@ int get_last_nonblank(const char *s)
     int i;
 
     i = (int)strlen(s) - 1;
-    while (i >= 0 && (s[i] == ' ' || s[i] == '\t'))
+    while (i >= 0 && is_blank(s[i]))
     {
 	i--;
     }
@@ -623,7 +647,7 @@ char *trim_string(const char *s)
  */
 
 int get_next_cmd(FILE *f, char **cmd, char **arg, int *empty_line, int *eof,
-	char *errstr)
+	char **errstr)
 {
     char line[LINEBUFSIZE];
     char *p;
@@ -638,7 +662,7 @@ int get_next_cmd(FILE *f, char **cmd, char **arg, int *empty_line, int *eof,
     {
 	if (ferror(f))
 	{
-	    snprintf(errstr, errstr_bufsize, "input error");
+	    *errstr = xasprintf(_("input error"));
 	    return CONF_EIO;
 	}
 	else /* EOF */
@@ -652,11 +676,16 @@ int get_next_cmd(FILE *f, char **cmd, char **arg, int *empty_line, int *eof,
     if ((p = strchr(line, '\n')))
     {
 	*p = '\0';
+	/* Kill '\r' (if CRLF line endings are used) */
+	if (p > line && *(p - 1) == '\r')
+	{
+	    *(p - 1) = '\0';
+	}
     }
     else if (strlen(line) == LINEBUFSIZE - 1)
     {
-	snprintf(errstr, errstr_bufsize, 
-		"line longer than %d characters", LINEBUFSIZE - 1);
+	*errstr = xasprintf(_("line longer than %d characters"),
+		LINEBUFSIZE - 1);
 	return CONF_EPARSE;
     }
 
@@ -688,7 +717,7 @@ int get_next_cmd(FILE *f, char **cmd, char **arg, int *empty_line, int *eof,
  */
 
 int read_account_list(int line, list_t *acc_list, char *s, list_t *l,
-	char *errstr)
+	char **errstr)
 {
     list_t *lp = l;
     char *comma;
@@ -706,14 +735,13 @@ int read_account_list(int line, list_t *acc_list, char *s, list_t *l,
 	if (*acc_id == '\0')
 	{
 	    free(acc_id);
-	    snprintf(errstr, errstr_bufsize,
-		    "line %d: missing account name", line);
+	    *errstr = xasprintf(_("line %d: missing account name"), line);
 	    return CONF_ESYNTAX;
 	}
 	if (!(acc = find_account(acc_list, acc_id)))
 	{
-	    snprintf(errstr, errstr_bufsize,
-		    "line %d: account %s not (yet) defined", line, acc_id);
+	    *errstr = xasprintf(_("line %d: account %s not (yet) defined"),
+		    line, acc_id);
 	    free(acc_id);
 	    return CONF_ESYNTAX;
 	}
@@ -745,7 +773,7 @@ int read_account_list(int line, list_t *acc_list, char *s, list_t *l,
  */
 
 int read_conffile(const char *conffile, FILE *f, list_t **acc_list, 
-	char *errstr)
+	char **errstr)
 {
     int e;
     list_t *p;
@@ -800,8 +828,8 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 	{
 	    if (*arg != '\0')
 	    {
-		snprintf(errstr, errstr_bufsize, 
-			"line %d: command %s does not take an argument", 
+		*errstr = xasprintf(
+			_("line %d: command %s does not take an argument"), 
 			line, cmd);
 		e = CONF_ESYNTAX;
 		break;
@@ -829,26 +857,24 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 	    if (*acc_id == '\0')
 	    {
 		list_free(copy_from);
-		snprintf(errstr, errstr_bufsize,
-			"line %d: missing account name", line);
+		*errstr = xasprintf(_("line %d: missing account name"), line);
 		e = CONF_ESYNTAX;
 		break;
 	    }
 	    if (strchr(acc_id, ':') || strchr(acc_id, ','))
 	    {
 		list_free(copy_from);
-		snprintf(errstr, errstr_bufsize,
-			"line %d: an account name must not contain "
- 			"colons or commas", line);
+		*errstr = xasprintf(_("line %d: an account name must not "
+			    "contain colons or commas"), line);
 		e = CONF_ESYNTAX;
 		break;
 	    }
 	    if (find_account(*acc_list, acc_id))
 	    {
 		list_free(copy_from);
-		snprintf(errstr, errstr_bufsize,
-			"line %d: account %s was already defined in this file",
-			line, arg);
+		*errstr = xasprintf(
+			_("line %d: account %s was already defined "
+			    "in this file"), line, arg);
 		e = CONF_ESYNTAX;
 		break;
 	    }
@@ -871,8 +897,8 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 	    acc->mask |= ACC_HOST;
 	    if (*arg == '\0')
 	    {
-		snprintf(errstr, errstr_bufsize, 
-			"line %d: command %s needs an argument", line, cmd);
+		*errstr = xasprintf(_("line %d: command %s needs an argument"),
+			line, cmd);
 		e = CONF_ESYNTAX;
 		break;
 	    }
@@ -890,8 +916,8 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 		/* We should go back to the default, which is to call
 		 * get_default_port(), but we cannot since the account is not
 		 * complete yet. So demand an argument here. */
-		snprintf(errstr, errstr_bufsize, 
-			"line %d: command %s needs an argument", line, cmd);
+		*errstr = xasprintf(_("line %d: command %s needs an argument"),
+			line, cmd);
 		e = CONF_ESYNTAX;
 		break;
 	    }
@@ -900,21 +926,25 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 		acc->port = get_pos_int(arg);
 		if (acc->port < 1 || acc->port > 65535)
 		{
-		    snprintf(errstr, errstr_bufsize,
- 			    "line %d: invalid argument %s for command %s",
+		    *errstr = xasprintf(
+			    _("line %d: invalid argument %s for command %s"),
 			    line, arg, cmd);
 		    e = CONF_ESYNTAX;
 		    break;
 		}
 	    }
 	}
-	else if (strcmp(cmd, "connect_timeout") == 0)
+	else if (strcmp(cmd, "timeout") == 0 
+		|| strcmp(cmd, "connect_timeout") == 0)
  	{
-	    acc->mask |= ACC_CONNECT_TIMEOUT;
+	    /* For compatibility with versions <= 1.4.1, connect_timeout is
+	     * accepted as an alias for timeout, though it had a slightly
+	     * different meaning. */
+	    acc->mask |= ACC_TIMEOUT;
 	    if (*arg == '\0')
 	    {
-		snprintf(errstr, errstr_bufsize,
-			"line %d: command %s needs an argument", line, cmd);
+		*errstr = xasprintf(_("line %d: command %s needs an argument"),
+			line, cmd);
 		e = CONF_ESYNTAX;
 		break;
 	    }
@@ -922,16 +952,15 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 	    {
 		if (is_off(arg))
 		{
-		    acc->connect_timeout = 0;
+		    acc->timeout = 0;
 		}
 		else
 		{
-		    acc->connect_timeout = get_pos_int(arg);
-		    if (acc->connect_timeout < 1)
+		    acc->timeout = get_pos_int(arg);
+		    if (acc->timeout < 1)
 		    {
-			snprintf(errstr, errstr_bufsize,
-				"line %d: invalid argument %s for command %s",
-				line, arg, cmd);
+			*errstr = xasprintf(_("line %d: invalid argument %s "
+				    "for command %s"), line, arg, cmd);
 			e = CONF_ESYNTAX;
 			break;
 		    }
@@ -943,8 +972,8 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 	    acc->mask |= ACC_PROTOCOL;
 	    if (*arg == '\0')
 	    {
-		snprintf(errstr, errstr_bufsize, 
-			"line %d: command %s needs an argument", line, cmd);
+		*errstr = xasprintf(_("line %d: command %s needs an argument"),
+			line, cmd);
 		e = CONF_ESYNTAX;
 		break;
 	    }
@@ -960,8 +989,8 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 		}
 		else
 		{
-		    snprintf(errstr, errstr_bufsize,
-			    "line %d: invalid argument %s for command %s", 
+		    *errstr = xasprintf(
+			    _("line %d: invalid argument %s for command %s"),
 			    line, arg, cmd);
 		    e = CONF_ESYNTAX;
 		    break;
@@ -974,17 +1003,37 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 	    free(acc->domain);
 	    acc->domain = xstrdup(arg);
 	}
+	else if (strcmp(cmd, "auto_from") == 0)
+	{
+	    acc->mask |= ACC_AUTO_FROM;
+	    if (*arg == '\0' || is_on(arg))
+	    {
+		acc->auto_from = 1;
+	    }
+	    else if (is_off(arg))
+	    {
+		acc->auto_from = 0;
+	    }
+	    else
+	    {
+	    	*errstr = xasprintf(
+    			_("line %d: invalid argument %s for command %s"),
+			line, arg, cmd);
+		e = CONF_ESYNTAX;
+		break;
+	    }	    
+	}
 	else if (strcmp(cmd, "from") == 0)
 	{
 	    acc->mask |= ACC_FROM;
 	    free(acc->from);
-	    acc->from = (*arg == '\0') ? NULL : xstrdup(arg);
+	    acc->from = xstrdup(arg);
 	}
 	else if (strcmp(cmd, "maildomain") == 0)
 	{
 	    acc->mask |= ACC_MAILDOMAIN;
 	    free(acc->maildomain);
-	    acc->maildomain = (*arg == '\0') ? NULL : xstrdup(arg);
+	    acc->maildomain = xstrdup(arg);
 	}
 	else if (strcmp(cmd, "auth") == 0)
 	{
@@ -1004,9 +1053,10 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 	    }
 	    else
 	    {
-		snprintf(errstr, errstr_bufsize, 
-			"line %d: invalid argument %s for command %s", 
-			line, arg, cmd);
+		acc->auth_mech = NULL;
+		*errstr = xasprintf(
+	    		_("line %d: invalid argument %s for command %s"),
+    			line, arg, cmd);
 		e = CONF_ESYNTAX;
 		break;
 	    }
@@ -1042,82 +1092,8 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 	    }
 	    else
 	    {
-		snprintf(errstr, errstr_bufsize, 
-			"line %d: invalid argument %s for command %s", 
-			line, arg, cmd);
-		e = CONF_ESYNTAX;
-		break;
-	    }
-	}
-	else if (strcmp(cmd, "tls_key_file") == 0)
-	{
-	    acc->mask |= ACC_TLS_KEY_FILE;
-	    free(acc->tls_key_file);
-	    if (*arg == '\0')
-	    {
-		acc->tls_key_file = NULL;
-	    }
-	    else
-	    {
-		if (expand_tilde(arg, &(acc->tls_key_file), errstr) 
-			!= PATHS_EOK)
-		{
-		    e = CONF_ESYNTAX;
-		    break;
-		}
-	    }
-	}
-	else if (strcmp(cmd, "tls_cert_file") == 0)
-	{
-	    acc->mask |= ACC_TLS_CERT_FILE;
-	    free(acc->tls_cert_file);
-	    if (*arg == '\0')
-	    {
-		acc->tls_cert_file = NULL;
-	    }
-	    else
-	    {
-		if (expand_tilde(arg, &(acc->tls_cert_file), errstr) 
-			!= PATHS_EOK)
-		{
-		    e = CONF_ESYNTAX;
-		    break;
-		}
-	    }
-	}
-	else if (strcmp(cmd, "tls_trust_file") == 0)
-	{
-	    acc->mask |= ACC_TLS_TRUST_FILE;
-	    free(acc->tls_trust_file);
-	    if (*arg == '\0')
-	    {
-		acc->tls_trust_file = NULL;
-	    }
-	    else
-	    {
-		if (expand_tilde(arg, &(acc->tls_trust_file), errstr) 
-			!= PATHS_EOK)
-		{
-		    e = CONF_ESYNTAX;
-		    break;
-		}
-	    }
-	}
-	else if (strcmp(cmd, "tls_certcheck") == 0)
-	{
-	    acc->mask |= ACC_TLS_NOCERTCHECK;
-	    if (*arg == '\0' || is_on(arg))
-	    {
-		acc->tls_nocertcheck = 0;
-	    }
-	    else if (is_off(arg))
-	    {
-		acc->tls_nocertcheck = 1;
-	    }
-	    else
-	    {
-		snprintf(errstr, errstr_bufsize, 
-			"line %d: invalid argument %s for command %s", 
+	    	*errstr = xasprintf(
+    			_("line %d: invalid argument %s for command %s"),
 			line, arg, cmd);
 		e = CONF_ESYNTAX;
 		break;
@@ -1136,8 +1112,87 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 	    }
 	    else
 	    {
-		snprintf(errstr, errstr_bufsize, 
-			"line %d: invalid argument %s for command %s", 
+	    	*errstr = xasprintf(
+    			_("line %d: invalid argument %s for command %s"),
+			line, arg, cmd);
+		e = CONF_ESYNTAX;
+		break;
+	    }
+	}
+	else if (strcmp(cmd, "tls_key_file") == 0)
+	{
+	    acc->mask |= ACC_TLS_KEY_FILE;
+	    free(acc->tls_key_file);
+	    if (*arg == '\0')
+	    {
+		acc->tls_key_file = NULL;
+	    }
+	    else
+	    {
+		acc->tls_key_file = expand_tilde(arg);
+	    }
+	}
+	else if (strcmp(cmd, "tls_cert_file") == 0)
+	{
+	    acc->mask |= ACC_TLS_CERT_FILE;
+	    free(acc->tls_cert_file);
+	    if (*arg == '\0')
+	    {
+		acc->tls_cert_file = NULL;
+	    }
+	    else
+	    {
+		acc->tls_cert_file = expand_tilde(arg);
+	    }
+	}
+	else if (strcmp(cmd, "tls_trust_file") == 0)
+	{
+	    acc->mask |= ACC_TLS_TRUST_FILE;
+	    free(acc->tls_trust_file);
+	    if (*arg == '\0')
+	    {
+		acc->tls_trust_file = NULL;
+	    }
+	    else
+	    {
+		acc->tls_trust_file = expand_tilde(arg);
+	    }
+	}
+	else if (strcmp(cmd, "tls_certcheck") == 0)
+	{
+	    acc->mask |= ACC_TLS_NOCERTCHECK;
+	    if (*arg == '\0' || is_on(arg))
+	    {
+		acc->tls_nocertcheck = 0;
+	    }
+	    else if (is_off(arg))
+	    {
+		acc->tls_nocertcheck = 1;
+	    }
+	    else
+	    {
+	    	*errstr = xasprintf(
+    			_("line %d: invalid argument %s for command %s"),
+			line, arg, cmd);
+		e = CONF_ESYNTAX;
+		break;
+	    }
+	}
+	else if (strcmp(cmd, "tls_force_sslv3") == 0)
+	{
+	    acc->mask |= ACC_TLS_FORCE_SSLV3;
+	    if (*arg == '\0' || is_on(arg))
+	    {
+		acc->tls_force_sslv3 = 1;
+	    }
+	    else if (is_off(arg))
+	    {
+		acc->tls_force_sslv3 = 0;
+	    }
+	    else
+	    {
+	    	*errstr = xasprintf(
+    			_("line %d: invalid argument %s for command %s"),
 			line, arg, cmd);
 		e = CONF_ESYNTAX;
 		break;
@@ -1149,14 +1204,18 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 	    free(acc->dsn_return);
 	    if (*arg == '\0')
 	    {
-		snprintf(errstr, errstr_bufsize, 
-			"line %d: command %s needs an argument", line, cmd);
+		*errstr = xasprintf(_("line %d: command %s needs an argument"),
+			line, cmd);
 		e = CONF_ESYNTAX;
 		break;
 	    }
 	    else
 	    {
-		if (strcmp(arg, "headers") == 0)
+		if (is_off(arg))
+		{
+		    acc->dsn_return = NULL;
+		}
+		else if (strcmp(arg, "headers") == 0)
 		{
 		    acc->dsn_return = xstrdup("HDRS");
 		}
@@ -1166,8 +1225,8 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 		}
 		else
 		{
-		    snprintf(errstr, errstr_bufsize,
-			    "line %d: invalid argument %s for command %s", 
+		    *errstr = xasprintf(
+			    _("line %d: invalid argument %s for command %s"),
 			    line, arg, cmd);
 		    e = CONF_ESYNTAX;
 		    break;
@@ -1180,22 +1239,29 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 	    free(acc->dsn_notify);
 	    if (*arg == '\0')
 	    {
-		snprintf(errstr, errstr_bufsize, 
-			"line %d: command %s needs an argument", line, cmd);
+		*errstr = xasprintf(_("line %d: command %s needs an argument"),
+			line, cmd);
 		e = CONF_ESYNTAX;
 		break;
 	    }
 	    else
 	    {
-		if (check_dsn_notify_arg(arg) != 0)
+		if (is_off(arg))
 		{
-		    snprintf(errstr, errstr_bufsize,
-			    "line %d: invalid argument %s for command %s", 
+		    acc->dsn_notify = NULL;
+		}
+		else if (check_dsn_notify_arg(arg) == 0)
+		{
+		    acc->dsn_notify = xstrdup(arg);
+		}
+		else
+		{
+		    *errstr = xasprintf(
+			    _("line %d: invalid argument %s for command %s"),
 			    line, arg, cmd);
 		    e = CONF_ESYNTAX;
 		    break;
 		}
-		acc->dsn_notify = xstrdup(arg);
 	    }
 	}
 	else if (strcmp(cmd, "keepbcc") == 0)
@@ -1211,8 +1277,8 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 	    }
 	    else
 	    {
-	   	snprintf(errstr, errstr_bufsize,
-    			"line %d: invalid argument %s for command %s", 
+    		*errstr = xasprintf(
+			_("line %d: invalid argument %s for command %s"),
 			line, arg, cmd);
 		e = CONF_ESYNTAX;
 		break;
@@ -1228,11 +1294,7 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 	    }
 	    else
 	    {
-		if (expand_tilde(arg, &(acc->logfile), errstr) != PATHS_EOK)
-		{
-		    e = CONF_ESYNTAX;
-		    break;
-		}
+		acc->logfile = expand_tilde(arg);
 	    }
 	}
 	else if (strcmp(cmd, "syslog") == 0)
@@ -1251,8 +1313,8 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 	    {
 		if (check_syslog_arg(arg) != 0)
 		{
-		    snprintf(errstr, errstr_bufsize,
-		    	    "line %d: invalid argument %s for command %s", 
+		    *errstr = xasprintf(
+			    _("line %d: invalid argument %s for command %s"),
 			    line, arg, cmd);
 		    e = CONF_ESYNTAX;
 		    break;
@@ -1266,8 +1328,8 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 	    acc->mask |= ACC_TLS_NOCERTCHECK;
 	    if (*arg != '\0')
 	    {
-		snprintf(errstr, errstr_bufsize, 
-			"line %d: command %s does not take an argument", 
+		*errstr = xasprintf(
+			_("line %d: command %s does not take an argument"), 
 			line, cmd);
 		e = CONF_ESYNTAX;
 		break;
@@ -1283,8 +1345,8 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 	    acc->mask |= ACC_TLS_NOSTARTTLS;
 	    if (*arg != '\0')
 	    {
-		snprintf(errstr, errstr_bufsize, 
-			"line %d: command %s does not take an argument", 
+		*errstr = xasprintf(
+			_("line %d: command %s does not take an argument"), 
 			line, cmd);
 		e = CONF_ESYNTAX;
 		break;
@@ -1296,8 +1358,7 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 	}
 	else
 	{
-	    snprintf(errstr, errstr_bufsize, 
-		    "line %d: unknown command %s", line, cmd);
+	    *errstr = xasprintf(_("line %d: unknown command %s"), line, cmd);
 	    e = CONF_ESYNTAX;
 	    break;
 	}
@@ -1325,25 +1386,43 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
  */
 
 int get_conf(const char *conffile, int securitycheck, list_t **acc_list, 
-	char *errstr)
+	char **errstr)
 {
     FILE *f;
     int e;
 
     if (!(f = fopen(conffile, "r")))
     {
-	snprintf(errstr, errstr_bufsize, "%s", strerror(errno));
+	*errstr = xasprintf("%s", strerror(errno));
 	return CONF_ECANTOPEN;
     }
     if (securitycheck)
     {
-	/* because we do the check after we opened the file, we know that it
-	 * exists, so we can treat all error codes returned by check_secure() as
-	 * PATHS_EINSECURE. */
-	if (check_secure(conffile, errstr) != PATHS_EOK)
+	switch (check_secure(conffile))
+	{
+	    case 1:
+		*errstr = xasprintf(_("must be owned by you"));
+		e = CONF_EINSECURE;
+		break;
+
+	    case 2:
+		*errstr = xasprintf(_("must have no more than user "
+			    "read/write permissions"));
+		e = CONF_EINSECURE;
+		break;
+
+	    case 3:
+		*errstr = xasprintf("%s", strerror(errno));
+		e = CONF_EIO;
+		break;
+
+	    default:
+		e = CONF_EOK;
+	}
+	if (e != CONF_EOK)
 	{
 	    fclose(f);
-	    return CONF_EINSECURE;
+	    return e;
 	}
     }
     if ((e = read_conffile(conffile, f, acc_list, errstr)) 
