@@ -3,7 +3,7 @@
  * 
  * This file is part of msmtp, an SMTP client.
  *
- * Copyright (C) 2000, 2003, 2004, 2005, 2006
+ * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007
  * Martin Lambers <marlam@marlam.de>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -23,49 +23,41 @@
 
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+# include "config.h"
 #endif
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#ifdef W32_NATIVE
-#define WINVER 0x0501
-#include <winsock2.h>
-#include <ws2tcpip.h>
-#elif defined (DJGPP)
 #include <unistd.h>
-#include <tcp.h>
 #include <fcntl.h>
-#include <netdb.h>
-#ifndef NI_MAXHOST
-#define NI_MAXHOST 1025
-#endif
 #include <errno.h>
-extern int errno;
-#else /* UNIX */
-#include <unistd.h>
-#include <fcntl.h>
-#ifndef HAVE_GETADDRINFO
-#include <netinet/in.h>
-extern int h_errno;
-#endif
 #include <sys/time.h>
 #include <sys/types.h>
-#include <sys/socket.h>
-#include <arpa/inet.h>
-#include <netdb.h>
-#include <errno.h>
-extern int errno;
-#endif /* UNIX */
+#ifdef W32_NATIVE
+# define WINVER 0x0501
+# include <winsock2.h>
+# include <ws2tcpip.h>
+#else
+# ifndef HAVE_GETADDRINFO
+#  include <netinet/in.h>
+   extern int h_errno;
+# endif
+# include <sys/socket.h>
+# include <arpa/inet.h>
+# include <netdb.h>
+# ifndef NI_MAXHOST
+#  define NI_MAXHOST 1025
+# endif
+#endif
 
-#ifdef USE_LIBIDN
-#include <idna.h>
+#ifdef HAVE_LIBIDN
+# include <idna.h>
 #endif
 
 #include "gettext.h"
-#include "xvasprintf.h"
 #include "xalloc.h"
+#include "xvasprintf.h"
 
 #include "net.h"
 
@@ -364,7 +356,7 @@ int net_open_socket(const char *hostname, int port, int timeout, int *ret_fd,
     int saved_errno;
     int cause;
     char nameinfo_buffer[NI_MAXHOST];
-#ifdef USE_LIBIDN
+#ifdef HAVE_LIBIDN
     char *hostname_ascii;
 #endif
     
@@ -377,7 +369,7 @@ int net_open_socket(const char *hostname, int port, int timeout, int *ret_fd,
     hints.ai_addr = NULL;
     hints.ai_next = NULL;
     port_string = xasprintf("%d", port);
-#ifdef USE_LIBIDN
+#ifdef HAVE_LIBIDN
     if (idna_to_ascii_lz(hostname, &hostname_ascii, 0) != IDNA_SUCCESS)
     {
 	hostname_ascii = xstrdup(hostname);
@@ -507,7 +499,7 @@ int net_open_socket(const char *hostname, int port, int timeout, int *ret_fd,
 #endif /* W32_NATIVE */
     struct in_addr addr;
     char *p;
-#ifdef USE_LIBIDN
+#ifdef HAVE_LIBIDN
     char *hostname_ascii;
 #endif
     
@@ -524,7 +516,7 @@ int net_open_socket(const char *hostname, int port, int timeout, int *ret_fd,
     else
     {
 #endif /* W32_NATIVE */
-#ifdef USE_LIBIDN
+#ifdef HAVE_LIBIDN
 	if (idna_to_ascii_lz(hostname, &hostname_ascii, 0) != IDNA_SUCCESS)
 	{
 	    hostname_ascii = xstrdup(hostname);
