@@ -3,7 +3,7 @@
  *
  * This file is part of msmtp, an SMTP client.
  *
- * Copyright (C) 2000, 2003, 2004, 2005
+ * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007
  * Martin Lambers <marlam@marlam.de>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -22,7 +22,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+# include "config.h"
 #endif
 
 #include <stdio.h>
@@ -33,13 +33,12 @@
 #include <strings.h>
 #include <ctype.h>
 #include <errno.h>
-extern int errno;
 
-#ifdef USE_GSASL
-#include <gsasl.h>
+#ifdef HAVE_LIBGSASL
+# include <gsasl.h>
 #else
-#include "base64.h"
-#include "crypto.h"
+# include "base64.h"
+# include "hmac.h"
 #endif
 
 #include "gettext.h"
@@ -48,11 +47,11 @@ extern int errno;
 
 #include "list.h"
 #include "net.h"
-#ifdef HAVE_SSL
-#include "tls.h"
-#endif /* HAVE_SSL */
-#include "stream.h"
 #include "smtp.h"
+#include "stream.h"
+#ifdef HAVE_TLS
+#include "tls.h"
+#endif /* HAVE_TLS */
 
 
 /* This defines the maximum number of lines in a multiline server reply.
@@ -109,9 +108,9 @@ smtp_server_t smtp_new(FILE *debug, int protocol)
 
     srv.fd = -1;
     net_readbuf_init(&(srv.readbuf));
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     tls_clear(&srv.tls);
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
     srv.protocol = protocol;
     srv.cap.flags = 0;
     srv.cap.size = 0;
@@ -164,7 +163,7 @@ int smtp_get_msg(smtp_server_t *srv, list_t **msg, char **errstr)
     counter = 0;
     do
     {
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
 	if (tls_is_active(&srv->tls))
 	{
 	    if (tls_gets(&srv->tls, line, SMTP_BUFSIZE, &len, errstr) 
@@ -176,16 +175,16 @@ int smtp_get_msg(smtp_server_t *srv, list_t **msg, char **errstr)
 	}
 	else
 	{
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 	    if (net_gets(srv->fd, &(srv->readbuf), line, SMTP_BUFSIZE, &len, 
 			errstr) != NET_EOK)
 	    {
 		list_xfree(l, free);
 		return SMTP_EIO;
 	    }
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
 	}
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 	if (len < 4 
 		|| !(isdigit((unsigned char)line[0]) 
 		    && isdigit((unsigned char)line[1]) 
@@ -262,18 +261,18 @@ int smtp_put(smtp_server_t *srv, const char *s, size_t len, char **errstr)
 {
     int e = 0;
 
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     if (tls_is_active(&srv->tls))
     {
 	e = (tls_puts(&srv->tls, s, len, errstr) != TLS_EOK);
     }
     else
     {
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 	e = (net_puts(srv->fd, s, len, errstr) != NET_EOK);
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
     if (e)
     {
 	return SMTP_EIO;
@@ -529,14 +528,15 @@ int smtp_init(smtp_server_t *srv, const char *ehlo_domain, list_t **errmsg,
  * see smtp.h
  */
 
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
 int smtp_tls_init(smtp_server_t *srv, const char *tls_key_file, 
-	const char *tls_ca_file, const char *tls_trust_file, char **errstr)
+	const char *tls_ca_file, const char *tls_trust_file, 
+	int force_sslv3, char **errstr)
 {
     return tls_init(&srv->tls, tls_key_file, tls_ca_file, tls_trust_file, 
-	    errstr);
+	    force_sslv3, errstr);
 }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 
 
 /*
@@ -545,7 +545,7 @@ int smtp_tls_init(smtp_server_t *srv, const char *tls_key_file,
  * see smtp.h
  */
 
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
 int smtp_tls_starttls(smtp_server_t *srv, list_t **error_msg, char **errstr)
 {
     int e;
@@ -569,7 +569,7 @@ int smtp_tls_starttls(smtp_server_t *srv, list_t **error_msg, char **errstr)
     list_xfree(msg, free);
     return SMTP_EOK;
 }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 
 
 /*
@@ -578,14 +578,14 @@ int smtp_tls_starttls(smtp_server_t *srv, list_t **error_msg, char **errstr)
  * see smtp.h
  */
 
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
 int smtp_tls(smtp_server_t *srv, const char *hostname, int tls_nocertcheck,
 	tls_cert_info_t *tci, char **errstr)
 {
     return tls_start(&srv->tls, srv->fd, hostname, tls_nocertcheck, tci, 
 	    errstr);
 }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 
 
 /*
@@ -596,7 +596,7 @@ int smtp_tls(smtp_server_t *srv, const char *hostname, int tls_nocertcheck,
  * Used error codes: SMTP_EIO, SMTP_EPROTO, SMTP_EAUTHFAIL, SMTP_EINVAL
  */
 
-#ifndef USE_GSASL
+#ifndef HAVE_LIBGSASL
 int smtp_auth_plain(smtp_server_t *srv, const char *user, const char *password,
 	list_t **error_msg, char **errstr)
 {
@@ -650,7 +650,7 @@ int smtp_auth_plain(smtp_server_t *srv, const char *user, const char *password,
 
     return SMTP_EOK;
 }
-#endif /* !USE_GSASL */
+#endif /* !HAVE_LIBGSASL */
 
 
 /*
@@ -661,7 +661,7 @@ int smtp_auth_plain(smtp_server_t *srv, const char *user, const char *password,
  * Used error codes: SMTP_EIO, SMTP_EPROTO, SMTP_EAUTHFAIL, SMTP_EINVAL
  */
 
-#ifndef USE_GSASL
+#ifndef HAVE_LIBGSASL
 int smtp_auth_login(smtp_server_t *srv, const char *user, const char *password,
 	list_t **error_msg, char **errstr)
 {
@@ -733,7 +733,7 @@ int smtp_auth_login(smtp_server_t *srv, const char *user, const char *password,
 
     return SMTP_EOK;
 }
-#endif /* !USE_GSASL */
+#endif /* !HAVE_LIBGSASL */
 
 
 /*
@@ -744,7 +744,7 @@ int smtp_auth_login(smtp_server_t *srv, const char *user, const char *password,
  * Used error codes: SMTP_EIO, SMTP_EPROTO, SMTP_EAUTHFAIL, SMTP_EINVAL
  */
 
-#ifndef USE_GSASL
+#ifndef HAVE_LIBGSASL
 int smtp_auth_cram_md5(smtp_server_t *srv, const char *user, 
 	const char *password,
 	list_t **error_msg, char **errstr)
@@ -790,7 +790,7 @@ int smtp_auth_cram_md5(smtp_server_t *srv, const char *user,
 	return SMTP_EPROTO;
     }
     list_xfree(msg, free);
-    md5_hmac(password, strlen(password), b64, len, digest);
+    hmac_md5(password, strlen(password), b64, len, digest);
     free(b64);
     
     /* construct username + ' ' + digest_in_hex */
@@ -830,7 +830,7 @@ int smtp_auth_cram_md5(smtp_server_t *srv, const char *user,
 
     return SMTP_EOK;
 }
-#endif /* !USE_GSASL */
+#endif /* !HAVE_LIBGSASL */
 
 
 /*
@@ -843,7 +843,7 @@ int smtp_auth_cram_md5(smtp_server_t *srv, const char *user,
  * Used error codes: SMTP_EIO, SMTP_EPROTO, SMTP_EAUTHFAIL, SMTP_EINVAL
  */
 
-#ifndef USE_GSASL
+#ifndef HAVE_LIBGSASL
 int smtp_auth_external(smtp_server_t *srv, const char *user,
 	list_t **error_msg, char **errstr)
 {
@@ -893,7 +893,7 @@ int smtp_auth_external(smtp_server_t *srv, const char *user,
 
     return SMTP_EOK;
 }
-#endif /* !USE_GSASL */
+#endif /* !HAVE_LIBGSASL */
 
 
 /*
@@ -929,7 +929,7 @@ int smtp_server_supports_authmech(smtp_server_t *srv, const char *mech)
 
 int smtp_client_supports_authmech(const char *mech)
 {
-#ifdef USE_GSASL
+#ifdef HAVE_LIBGSASL
 
     int supported = 0;
     Gsasl *ctx;
@@ -942,14 +942,14 @@ int smtp_client_supports_authmech(const char *mech)
     gsasl_done(ctx);
     return supported;
     
-#else /* not USE_GSASL */
+#else /* not HAVE_LIBGSASL */
     
     return (strcmp(mech, "CRAM-MD5") == 0
 	    || strcmp(mech, "PLAIN") == 0
 	    || strcmp(mech, "EXTERNAL") == 0
 	    || strcmp(mech, "LOGIN") == 0);
     
-#endif /* not USE_GSASL */
+#endif /* not HAVE_LIBGSASL */
 }
 
 
@@ -963,7 +963,7 @@ int smtp_auth(smtp_server_t *srv,
 	const char *hostname,
 	const char *user, 
 	const char *password,
-#ifdef USE_GSASL
+#ifdef HAVE_LIBGSASL
 	const char *ntlmdomain,
 #else
 	const char *ntlmdomain UNUSED,
@@ -973,7 +973,7 @@ int smtp_auth(smtp_server_t *srv,
 	list_t **error_msg,
 	char **errstr)
 {
-#ifdef USE_GSASL
+#ifdef HAVE_LIBGSASL
     int e;
     list_t *msg;
     Gsasl *ctx;
@@ -983,7 +983,7 @@ int smtp_auth(smtp_server_t *srv,
     char *outbuf;
     int error_code;
     int auth_plain_special;
-    char *callback_password;
+    char *callback_password = NULL;
 
 
     *error_msg = NULL;
@@ -1032,7 +1032,7 @@ int smtp_auth(smtp_server_t *srv,
 	{
 	    auth_mech = "NTLM";
 	}
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
 	else if (tls_is_active(&srv->tls))
 	{
 	    if (gsasl_client_support_p(ctx, "PLAIN") 
@@ -1046,27 +1046,55 @@ int smtp_auth(smtp_server_t *srv,
 		auth_mech = "LOGIN";
 	    }
 	}
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
     }
     if (strcmp(auth_mech, "") == 0)
     {
 	gsasl_done(ctx);
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
 	if (!tls_is_active(&srv->tls))
 	{
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 	    *errstr = xasprintf(_("cannot use a secure authentication method"));
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
 	}
 	else
 	{
 	    *errstr = xasprintf(
 		    _("cannot find a usable authentication method"));
      	}
-#endif /* not HAVE_SSL */
+#endif /* not HAVE_TLS */
 	return SMTP_EUNAVAIL;
     }
     
+    /* Check availability of required authentication data */
+    if (strcmp(auth_mech, "EXTERNAL") != 0)
+    {
+	/* GSSAPI, DIGEST-MD5, CRAM-MD5, PLAIN, LOGIN, NTLM all need a user 
+	 * name */
+	if (!user)
+	{
+	    gsasl_done(ctx);
+	    *errstr = xasprintf(_("authentication method %s needs a user name"),
+		    auth_mech);
+	    return SMTP_EUNAVAIL;
+	}
+	/* DIGEST-MD5, CRAM-MD5, PLAIN, LOGIN, NTLM all need a password */
+	if (strcmp(auth_mech, "GSSAPI") != 0 && !password)
+	{
+	    if (!password_callback 
+		    || !(callback_password = password_callback(hostname, user)))
+	    {
+		gsasl_done(ctx);
+		*errstr = xasprintf(
+			_("authentication method %s needs a password"),
+			auth_mech);
+		return SMTP_EUNAVAIL;
+	    }
+	    password = callback_password;
+	}
+    }
+
     if ((error_code = gsasl_client_start(ctx, auth_mech, &sctx)) != GSASL_OK)
     {
 	gsasl_done(ctx);
@@ -1090,22 +1118,7 @@ int smtp_auth(smtp_server_t *srv,
     {
 	gsasl_property_set(sctx, GSASL_PASSWORD, password);
     }
-    /* If the callback fails, or there is none, leave the error message about a
-     * missing password to GSASL. */
-    else if (password_callback)
-    {
-	if (strcmp(auth_mech, "GSSAPI") != 0
-		&& strcmp(auth_mech, "EXTERNAL") != 0)
-	{
-	    /* All others (PLAIN, LOGIN, CRAM-MD5, DIGEST-MD5, NTLM) need a
-	     * password. */
-	    if ((callback_password = password_callback(hostname, user)))
-	    {
-		gsasl_property_set(sctx, GSASL_PASSWORD, callback_password);
-		free(callback_password);
-	    }	    
-	}
-    }
+    free(callback_password);
     /* For DIGEST-MD5 and GSSAPI */
     gsasl_property_set(sctx, GSASL_SERVICE, "smtp");
     if (hostname)
@@ -1258,7 +1271,7 @@ int smtp_auth(smtp_server_t *srv,
     }
     return SMTP_EOK;
 
-#else /* not USE_GSASL */
+#else /* not HAVE_LIBGSASL */
 
     char *callback_password = NULL;
     int e;
@@ -1280,7 +1293,7 @@ int smtp_auth(smtp_server_t *srv,
 	{
 	    auth_mech = "CRAM-MD5";
 	}
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
 	else if (tls_is_active(&srv->tls))
 	{
 	    if (srv->cap.flags & SMTP_CAP_AUTH_PLAIN)
@@ -1292,23 +1305,23 @@ int smtp_auth(smtp_server_t *srv,
 		auth_mech = "LOGIN";
 	    }
 	}
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
     }
     if (strcmp(auth_mech, "") == 0)
     {
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
        	if (!tls_is_active(&srv->tls))
  	{
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 	    *errstr = xasprintf(_("cannot use a secure authentication method"));
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     	}
     	else
  	{
 	    *errstr = xasprintf(
 		    _("cannot find a usable authentication method"));
 	}
-#endif /* not HAVE_SSL */
+#endif /* not HAVE_TLS */
 	return SMTP_EUNAVAIL;
     }
 
@@ -1360,7 +1373,7 @@ int smtp_auth(smtp_server_t *srv,
     free(callback_password);
     return e;
 
-#endif /* not USE_GSASL */
+#endif /* not HAVE_LIBGSASL */
 }
 
 
@@ -1856,11 +1869,11 @@ int smtp_quit(smtp_server_t *srv, char **errstr)
 
 void smtp_close(smtp_server_t *srv)
 {
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     if (tls_is_active(&srv->tls))
     {
 	tls_close(&srv->tls);
     }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
     net_close_socket(srv->fd);
 }
